<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/09/2004 *
 * ===================                          Last change: 10/31/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-list_unconfirmed.php                        *
 * -------------------------------------------------------------------- *
 * Short description : List unconfirmed mail links                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Unbest. Mail-Links auflisten                     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// Add description as navigation point
addYouAreHereLink('admin', __FILE__);

if (!isExtensionActive('mailid')) {
	displayMessage('{%pipe,generateExtensionInactiveNotInstalledMessage=mailid%}');
	return;
} // END - if

// Don't load the admin_list_unconfirmed template by default
$listed = false;

// Init query
$sql = '';

// List confirmation links from normal or bonus mails
if (isGetRequestParameterSet('mid')) {
	// SQL query for mail data
	$sql = sprintf("SELECT
	s.`id`, p.`sender`, p.`subject`, p.`text`, p.`url`, p.`timestamp`, s.`max_rec`
FROM
	`{?_MYSQL_PREFIX?}_pool` AS p
LEFT JOIN
	`{?_MYSQL_PREFIX?}_user_stats` AS s
ON
	p.id=s.pool_id
WHERE
	p.`id`=%s
LIMIT 1",
		bigintval(getRequestParameter('mid'))
	);

	// Column, type and id for member's mail
	$col = 'stats_id';
	$type = 'NORMAL';
	$ID = -1;

	// Load admin_list_unconfirmed template
	$listed = true;
	$DATA = getRequestParameter('mid');
	$mailType = 'mailid';
} elseif ((isGetRequestParameterSet('bid')) && (isExtensionActive('bonus'))) {
	// @TODO This constant might be unused? define('__LIST_UNCON_TITLE', '{--ADMIN_LIST_UNCONFIRMED_BONUS_LINKS--}');

	// SQL query for mail data (both ids are required for compatiblity to above normal mail
	$sql = sprintf("SELECT
	`id`,`id` AS `sender`,`subject`,`text`,`url`,`timestamp`,`mails_sent` AS `max_rec`
FROM
	`{?_MYSQL_PREFIX?}_bonus`
WHERE
	`id`=%s
LIMIT 1",
		bigintval(getRequestParameter('bid'))
	);

	// Column, type and id for member's mail
	$col = 'bonus_id';
	$type = 'BONUS';
	$ID = getRequestParameter('bid');

	// Load admin_list_unconfirmed template
	$listed = true;
	$DATA = $ID;
	$mailType = 'bonusid';
} else {
	// @TODO "Please do not call me directly." Should be rewritten to a nice selection depending on ext-bonus
	displayMessage('{--ADMIN_CALL_NOT_DIRECTLY--}');
}

// Shall I display links or not?
if (($listed === true) && (!empty($sql))) {
	// Load mail data
	$result_master = SQL_QUERY($sql, __FILE__, __LINE__);

	// Do we have an entry?
	if (SQL_NUMROWS($result_master) == 1) {
		// Mail order / bonus mail found
		$poolData = SQL_FETCHARRAY($result_master);

		// If there is a pool id and $ID is not set, we take it
		if (($poolData['id'] > 0) && ($ID == '-1')) {
			$ID = $poolData['id'];
		} // END - if

		// Bonus mails do always have a sender 'NULL'
		if ($col == 'bonus_id') {
			$poolData['sender'] = NULL;
		} // END - if

		// Load unconfirmed mail links. Hmmm, this select query is pretty cool
		// but it does only show unconfirmed mail links from existing user
		// accounts. So if you have delete one you did not see those links
		$result = SQL_QUERY_ESC("SELECT
	l.`userid`, u.`status`, u.`surname`, u.`family`, u.`gender`, u.`email`
FROM
	`{?_MYSQL_PREFIX?}_user_links` AS `l`
LEFT JOIN
	`{?_MYSQL_PREFIX?}_user_data` AS `u`
ON
	l.`userid`=u.`userid`
WHERE
	l.`%s`=%s
ORDER BY
	l.`userid` ASC
LIMIT %s",
			array(
				$col,
				$ID,
				bigintval($poolData['max_rec'])
			),__FILE__, __LINE__);

		// Do we have entries?
		if (!SQL_HASZERONUMS($result)) {
			// At least one link left to confirm
			$OUT = '';
			while ($row = SQL_FETCHARRAY($result)) {
				// User data found? We can take any field of u.
				if (!is_null($row['status'])) {
					// Prepare data for the row template
					$row = array(
						'userid' => $row['userid'],
						'link'   => $mailType,
						'id'     => $ID,
						'email'  => '<a href="' . generateEmailLink($row['email'], 'user_data') . '">{%pipe,translateGender=' . $row['gender'] . '%} ' . $row['surname'] . ' ' . $row['family'] . '</a>',
						'status' => $row['status'],
					);

					// Load row template and switch colors
					$OUT .= loadTemplate('admin_list_unconfirmed_row', true, $row);
				} else {
					// No user data found
					$OUT .= loadTemplate('admin_list_unconfirmed_row_404', true, $row);
				}
			} // END - while

			// Render it in our new listing
			$OUT = loadTemplate('admin_list_unconfirmed_list', true, $OUT);
		} else {
			// All links are confirmed... strange, you shall normally not get a link to this place in this scenario... hmmm.
			$OUT = displayMessage('{--ADMIN_UNCONFIRMED_NO_LINK_LEFT--}', true);
		}

		// Prepare content
		$content                = $poolData;
		$content['unconfirmed'] = SQL_NUMROWS($result);
		$content['timestamp']   = generateDateTime($poolData['timestamp'], 2);
		$content['rows']        = $OUT;

		// Free memory
		SQL_FREERESULT($result);

		// Load final template
		loadTemplate('admin_list_unconfirmed', false, $content);
	} elseif (getRequestParameter('mid') > 0) {
		// Data in pool or in user_stats not found, so let's find out where data is missing
		if (countSumTotalData(bigintval($ID), 'pool', 'id', 'id', true) == 1) {
			// pool table
			displayMessage('{%message,ADMIN_UNCONFIRMED_POOL_MISSING=' . $ID . '%}');
		} elseif (countSumTotalData(bigintval($ID), 'user_stats', 'id', 'pool_id', true) == 1) {
			// user_stats table
			displayMessage('{%message,ADMIN_UNCONFIRMED_STATS_MISSING=' . $ID . '%}');
		} else {
			// both or link is invalid
			displayMessage('{--ADMIN_UNCONFIRMED_INVALID_LINK--}');
		}
	} elseif (isGetRequestParameterSet('bid')) {
		// Data in bonus table not found
		displayMessage('{--ADMIN_UNCONFIRMED_INVALID_LINK--}');
	}

	// Free result
	SQL_FREERESULT($result_master);
} // END - if

// [EOF]
?>
