<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/25/2009 *
 * ===================                          Last change: 10/25/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : stats-functions.php                              *
 * -------------------------------------------------------------------- *
 * Short description : Automatical purging of outdated mail links       *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Auto-Loeschung von veralteten Mail-Links         *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Init stats system
function initStatsSystem () {
	// Is stats cache loaded?
	if ((!isset($GLOBALS['stats_loaded'])) && (ifStatsAreEnabled())) {
		// Init statistics array
		$GLOBALS['stats'] = array();

		// Load statistics entry from temporary table
		loadStatsTable();

		// Stats are loaded!
		$GLOBALS['stats_loaded'] = true;
	} // END - if
}

// Checks if we have a statistics entry
function isStatsEntrySet ($entry) {
	// Do we have the entry?
	return (isset($GLOBALS['stats'][$entry]));
}

// Increments a statistics entry
function incrementStatsEntry ($entry, $amount=1) {
	// Do we have stats enabled?
	if (!ifStatsAreEnabled()) return;

	// Is it there?
	if (isStatsEntrySet($entry)) {
		// Then increment it
		$GLOBALS['stats'][$entry] += $amount;
	} else {
		// Write it
		$GLOBALS['stats'][$entry] = $amount;
	}
}

// Getter for stats
function getStatsEntry ($entry) {
	// Default is zero
	$stats = '0';

	// Is it there?
	if (isStatsEntrySet($entry)) {
		// Then use it
		$stats = $GLOBALS['stats'][$entry];
	} // END - if

	// Return the value
	return $stats;
}

// Setter for stats
function setStatsEntry ($entry, $value) {
	$GLOBALS['stats'][$entry] = $value;
}

// Loads stats table
function loadStatsTable () {
	// Check if the link is up
	if (!SQL_IS_LINK_UP()) return false;

	// Do we have it there for today?
	if (!isStatsTableCreated()) {
		// Then create a default one
		createStatsTable();
	} // END - if

	// Load it from database
	$result = SQL_QUERY_ESC("SELECT `stats_entry`,`stats_value` FROM `{?_MYSQL_PREFIX?}_stats_%s` ORDER BY `stats_entry` ASC",
		array(
			generateDateTime(time(), '6')
		), __FUNCTION__, __LINE__);

	// Read all rows
	while ($row = SQL_FETCHARRAY($result)) {
		$GLOBALS['stats'][$row['stats_entry']] = $row['stats_value'];
	} // END - while

	// Free the result
	SQL_FREERESULT($result);
}

// Checks if the the statistics table is created
function isStatsTableCreated () {
	// Check if the link is up
	if (!SQL_IS_LINK_UP()) return false;

	// Ask for it
	$result = SQL_QUERY_ESC("SHOW TABLES LIKE '{?_MYSQL_PREFIX?}_stats_%s'",
		array(generateDateTime(time(), '6')), __FUNCTION__, __LINE__);

	// Do we have a row?
	return SQL_NUMROWS($result);
}

// Create the dummy table
function createStatsTable () {
	// Check if the link is up
	if (!SQL_IS_LINK_UP()) return false;

	// Create it here
	$result = SQL_QUERY_ESC("CREATE TEMPORARY TABLE IF NOT EXISTS `{?_MYSQL_PREFIX?}_stats_%s` (
`stats_entry` VARCHAR(100) NOT NULL DEFAULT '',
`stats_value` BIGINT(20) NOT NULL DEFAULT 0,
PRIMARY KEY (`stats_entry`)
) ENGINE = HEAP COMMENT = 'Temporary statistics table'",
		array(generateDateTime(time(), '6')), __FUNCTION__, __LINE__);
}

// Write all entries to the table
function writeStatsTable () {
	// Check if the link is up
	if (!SQL_IS_LINK_UP()) return false;

	// Empty the table first
	SQL_QUERY_ESC("TRUNCATE `{?_MYSQL_PREFIX?}_stats_%s`",
		array(generateDateTime(time(), '6')), __FUNCTION__, __LINE__);

	// Begin the SQL command
	$sql = sprintf("REPLACE INTO `{?_MYSQL_PREFIX?}_stats_%s` (`stats_entry`,`stats_value`) VALUES ",
		generateDateTime(time(), '6'));

	// Add all entries to the final query
	foreach ($GLOBALS['stats'] as $entry => $value) {
		$sql .= sprintf("('%s', %s),", $entry, bigintval($value));
	} // END - foreach

	// Finalize it and run it
	SQL_QUERY(substr($sql, 0, -1), __FUNCTION__, __LINE__);
}

// Filter for flushing statistics
function FILTER_FLUSH_STATS () {
	// Now do we have stats?
	if ((isset($GLOBALS['stats'])) && (!isInstallationPhase()) && (ifStatsAreEnabled())) {
		// Write statistics to temporary table
		writeStatsTable();
	} // END - if
}

// [EOF]
?>
